# NOTE:
#
#   This is an automatically generated Elasticgit Model definition
#   from an Avro schema. Do not manually edit this file unless you
#   absolutely know what you are doing.
#
# namespace: unicore.content.models
#
# VERSION INFO:
#
#   package:          elastic-git
#   package_version:  0.2.2
#   language_version: 2.7.6
#

from elasticgit import models


class Page(models.Model):
    
    subtitle = models.TextField("""subtitle""")
    description = models.TextField("""description""")
    language = models.TextField("""language""")
    title = models.TextField("""title""")
    created_at = models.TextField("""created_at""")
    published = models.BooleanField("""published""")
    modified_at = models.TextField("""modified_at""")
    linked_pages = models.ListField("""linked_pages""")
    slug = models.TextField("""slug""")
    content = models.TextField("""content""")
    source = models.TextField("""source""")
    featured = models.BooleanField("""featured""")
    featured_in_category = models.BooleanField("""featured_in_category""")
    id = models.TextField("""id""")
    position = models.IntegerField("""position""")
    primary_category = models.TextField("""primary_category""")
    uuid = models.UUIDField("""uuid""", fallbacks=[models.SingleFieldFallback('id'),])



class Category(models.Model):
    
    subtitle = models.TextField("""subtitle""")
    language = models.TextField("""language""")
    title = models.TextField("""title""")
    id = models.TextField("""id""")
    source = models.TextField("""source""")
    position = models.IntegerField("""position""")
    featured_in_navbar = models.BooleanField("""featured_in_navbar""")
    slug = models.TextField("""slug""")
    uuid = models.UUIDField("""uuid""", fallbacks=[models.SingleFieldFallback('id'),])



from elasticgit import EG


def push_to_git(repo_path, index_prefix):
    workspace = EG.workspace(repo_path,
                             index_prefix=index_prefix)
    if workspace.repo.remotes:
        repo = workspace.repo
        remote = repo.remote()
        remote.fetch()
        remote_master = remote.refs.master
        remote.push(remote_master.remote_head)



import json

from django.contrib import admin
from django.test.client import RequestFactory
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from cms.tests.base import BaseCmsTestCase
from cms.admin import (
    PostAdmin, CategoryAdmin, ContentRepositoryAdmin, PublishingTargetAdmin,
    CategoriesListFilter,
    PostSourceListFilter, CategorySourceListFilter,
    push_to_github, my_view)
from cms.models import Post, Category, ContentRepository, PublishingTarget

from unicore.content import models as eg_models


class BaseAdminTestCase(BaseCmsTestCase):

    def setUp(self):
        self.setUpWorkspace()

    def setUpWorkspace(self):
        self.workspace = self.mk_workspace()
        with self.active_workspace(self.workspace):
            self.category1 = Category.objects.create(title='category 1')
            self.category2 = Category.objects.create(title='category 2',
                                                     source=self.category1)
            self.category3 = Category.objects.create(title='category 3')

            self.post1 = Post.objects.create(title='post 1')
            self.post2 = Post.objects.create(title='post 2',
                                             primary_category=self.category1,
                                             source=self.post1)
            self.post3 = Post.objects.create(title='post 3',
                                             primary_category=self.category2)


class TestCategoriesListFilter(BaseAdminTestCase):

    def test_filter_knowncategory(self):
        filter = CategoriesListFilter(None, {
            'category_slug': self.category1.slug,
        }, Post, PostAdmin)
        [post2] = filter.queryset(None, Post.objects.all())
        self.assertEqual(post2, self.post2)

    def test_filter_unknown_category(self):
        filter = CategoriesListFilter(None, {
            'category_slug': 'foo',
        }, Post, PostAdmin)
        qs = filter.queryset(None, Post.objects.all())
        self.assertFalse(qs.exists())

    def test_filter_empty_category(self):
        filter = CategoriesListFilter(None, {
            'category_slug': self.category3.slug,
        }, Post, PostAdmin)
        qs = filter.queryset(None, Post.objects.all())
        self.assertFalse(qs.exists())

    def test_lookups(self):
        filter = CategoriesListFilter(None, {}, Post, PostAdmin)
        self.assertEqual([
            ('category-1', 'category 1'),
            ('category-2', 'category 2'),
            ('category-3', 'category 3'),
        ], list(filter.lookups(None, PostAdmin)))


class TestPostSourceListFilter(BaseAdminTestCase):

    def test_filter_source_slug(self):
        filter = PostSourceListFilter(None, {
            'source_slug': self.post1.slug,
        }, Post, PostAdmin)
        [post2] = filter.queryset(None, Post.objects.all())
        self.assertEqual(post2, self.post2)

    def test_filter_unknown_source(self):
        filter = PostSourceListFilter(None, {
            'source_slug': 'foo',
        }, Post, PostAdmin)
        qs = filter.queryset(None, Post.objects.all())
        self.assertFalse(qs.exists())

    def test_filter_empty_source(self):
        filter = PostSourceListFilter(None, {
            'source_slug': self.post2.slug,
        }, Post, PostAdmin)
        qs = filter.queryset(None, Post.objects.all())
        self.assertFalse(qs.exists())

    def test_lookups(self):
        filter = PostSourceListFilter(None, {}, Post, PostAdmin)
        self.assertEqual([
            ('post-1', 'post 1'),
        ], list(filter.lookups(None, PostAdmin)))


class TestCategorySourceListFilter(BaseAdminTestCase):

    def test_filter_source_slug(self):
        filter = CategorySourceListFilter(None, {
            'source_slug': self.category1.slug,
        }, Category, CategoryAdmin)
        [category2] = filter.queryset(None, Category.objects.all())
        self.assertEqual(category2, self.category2)

    def test_filter_unknown_source(self):
        filter = CategorySourceListFilter(None, {
            'source_slug': 'foo',
        }, Category, CategoryAdmin)
        qs = filter.queryset(None, Category.objects.all())
        self.assertFalse(qs.exists())

    def test_filter_empty_source(self):
        filter = CategorySourceListFilter(None, {
            'source_slug': self.category2.slug,
        }, Category, CategoryAdmin)
        qs = filter.queryset(None, Category.objects.all())
        self.assertFalse(qs.exists())

    def test_lookups(self):
        filter = CategorySourceListFilter(None, {}, Category, CategoryAdmin)
        self.assertEqual([
            ('category-1', 'category 1'),
        ], list(filter.lookups(None, CategoryAdmin)))


class TestPostAdmin(BaseAdminTestCase):

    def test_derivatives(self):
        post_admin = PostAdmin(Post, admin)
        self.assertEqual(post_admin._derivatives(self.post1), 1)
        self.assertEqual(post_admin._derivatives(self.post2), 0)

    def test_save_model(self):
        user = User.objects.create_user('foo', 'bar')
        request = RequestFactory().get('/')
        request.user = user

        post_admin = PostAdmin(Post, admin)
        post_admin.save_model(request, self.post1, None, None)
        saved_post = Post.objects.get(pk=self.post1.pk)
        self.assertEqual(saved_post.owner, user)
        self.assertEqual(saved_post.last_author, user)


class TestCategoryAdmin(BaseAdminTestCase):

    def test_derivatives(self):
        category_admin = CategoryAdmin(Category, admin)
        self.assertEqual(category_admin._derivatives(self.category1), 1)
        self.assertEqual(category_admin._derivatives(self.category2), 0)

    def test_save_model(self):
        user = User.objects.create_user('foo', 'bar')
        request = RequestFactory().get('/')
        request.user = user

        category_admin = CategoryAdmin(Category, admin)
        category_admin.save_model(request, self.category1, None, None)
        saved_category = Category.objects.get(pk=self.category1.pk)
        self.assertEqual(saved_category.last_author, user)


class TestContentRepositoryAdmin(BaseAdminTestCase):

    def test_get_object_side_effects(self):
        """
        NOTE: What we're testing here is that for any content repository
              we're always adding the default target if the Admin
              receives a `content repository` object to display
        """
        request = RequestFactory().get('/')

        repo_admin = ContentRepositoryAdmin(ContentRepository, admin)
        content_repo = ContentRepository.objects.create(name='the repo')
        self.assertFalse(content_repo.targets.exists())
        repo_admin.get_object(request, content_repo.pk)
        [target] = content_repo.targets.all()
        self.assertEqual(target, PublishingTarget.get_default_target())

    def test_has_add_permission(self):
        repo_admin = ContentRepositoryAdmin(ContentRepository, admin)
        self.assertTrue(repo_admin.has_add_permission())
        ContentRepository.objects.create(name='the repo')
        self.assertFalse(repo_admin.has_add_permission())


class TestPublishingTargetAdmin(BaseAdminTestCase):

    def test_has_add_permission(self):
        target_admin = PublishingTargetAdmin(PublishingTarget, admin)
        self.assertFalse(PublishingTarget.objects.exists())
        self.assertFalse(target_admin.has_add_permission())
        [target] = PublishingTarget.objects.all()
        self.assertEqual(target, PublishingTarget.get_default_target())


class TestCustomAdminViews(BaseAdminTestCase):

    def setUp(self):
        self.local_workspace = self.mk_workspace()
        self.remote_workspace = self.mk_workspace(
            name='%s_remote' % (self.id().lower(),),
            index_prefix='%s_remote' % (self.local_workspace.index_prefix,))
        self.create_pages(self.remote_workspace)
        local_repo = self.local_workspace.repo
        local_repo.create_remote('origin', self.remote_workspace.working_dir)
        self.local_workspace.fast_forward(
            branch_name='master', remote_name='origin')

    def do_push_to_github(self, request):
        # NOTE: In order to push to the remote workspace's master branch it
        #       needs to itself be checked out with a different branch,
        #       one cannot pushed to branches that are currently checked out.
        remote_repo = self.remote_workspace.repo
        remote_repo.git.checkout('HEAD', b='temp')

        with self.active_workspace(self.local_workspace):
            # make some changes locally
            self.setUpWorkspace()
            response = push_to_github(request)

        # NOTE: switching back to master on the remote because the changes
        #       should have been pushed and we are checking for their
        #       existence

        remote_repo.heads.master.checkout()
        self.remote_workspace.reindex(eg_models.Page)
        self.remote_workspace.refresh_index()
        self.assertEqual(
            self.remote_workspace.S(eg_models.Page).count(), 5)
        return response

    def test_push_to_github(self):
        request = RequestFactory().get('/')
        response = self.do_push_to_github(request)
        self.assertEqual(response['Location'], reverse('admin:index'))

    def test_push_to_github_ajax(self):
        request = RequestFactory().get(
            '/', HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        response = self.do_push_to_github(request)
        self.assertEqual(response['Content-Type'], 'application/json')
        self.assertEqual(json.loads(response.content), {
            'success': True
        })

    def test_view_github_configuration(self):
        with self.active_workspace(self.local_workspace):
            request = RequestFactory().get('/')
            response = my_view(request)
            for commit in self.local_workspace.repo.iter_commits():
                self.assertContains(response, commit.message)



import sys
from optparse import make_option

from django.core.management.base import BaseCommand
from django.core.exceptions import ValidationError
from django.utils.six.moves import input
from django.db.models.signals import post_save, post_delete
from django.conf import settings

from cms.models import (
    Post, Category, Localisation, auto_save_post_to_git,
    auto_save_category_to_git, auto_delete_post_to_git,
    auto_delete_category_to_git)

from elasticgit import EG

from unicore.content import models as eg_models

from html2text import html2text


class Command(BaseCommand):
    help = 'import all the current content from a github repository'

    option_list = BaseCommand.option_list + (
        make_option(
            '--quiet',
            action='store_true',
            dest='quiet',
            default=False,
            help='imports the data using default arguments'),
    )

    input_func = input

    def disconnect_signals(self):
        post_save.disconnect(auto_save_post_to_git, sender=Post)
        post_delete.disconnect(auto_delete_post_to_git, sender=Post)

        post_save.disconnect(auto_save_category_to_git, sender=Category)
        post_delete.disconnect(auto_delete_category_to_git, sender=Category)

    def reconnect_signals(self):
        post_save.connect(auto_save_post_to_git, sender=Post)
        post_delete.connect(auto_delete_post_to_git, sender=Post)

        post_save.connect(auto_save_category_to_git, sender=Category)
        post_delete.connect(auto_delete_category_to_git, sender=Category)

    def emit(self, message):
        if not self.quiet:
            self.stdout.write(message)

    def handle(self, *args, **options):
        self.disconnect_signals()
        self.quiet = options.get('quiet')
        workspace = EG.workspace(
            settings.GIT_REPO_PATH,
            index_prefix=settings.ELASTIC_GIT_INDEX_PREFIX)

        if not self.quiet:
            must_delete = self.get_input_data(
                'Do you want to delete existing data? Y/n: ', 'y')
        else:
            must_delete = 'y'

        if must_delete.lower() == 'y':
            self.emit('deleting existing content..')
            Post.objects.all().delete()
            Category.objects.all().delete()

        self.emit('creating categories..')
        categories = workspace.S(eg_models.Category).everything()

        for instance in categories:
            Category.objects.create(
                slug=instance.slug,
                title=instance.title,
                subtitle=instance.subtitle,
                localisation=Localisation._for(instance.language),
                featured_in_navbar=instance.featured_in_navbar or False,
                uuid=instance.uuid,
                position=instance.position,
            )

        # second pass to add related fields
        for instance in categories:
            if instance.source:
                c = Category.objects.get(uuid=instance.uuid)
                c.source = Category.objects.get(uuid=instance.source)
                c.save()

        # Manually refresh stuff because the command disables signals
        workspace.refresh_index()

        pages = workspace.S(eg_models.Page).everything()
        for instance in pages:
            primary_category = None
            if instance.primary_category:
                primary_category = Category.objects.get(
                    uuid=instance.primary_category)
            try:
                Post.objects.create(
                    title=instance.title,
                    subtitle=instance.subtitle,
                    slug=instance.slug,
                    description=instance.description,
                    content=html2text(instance.content),
                    created_at=instance.created_at,
                    modified_at=instance.modified_at,
                    featured_in_category=(
                        instance.featured_in_category or False),
                    featured=(
                        instance.featured or False),
                    localisation=Localisation._for(instance.language),
                    primary_category=primary_category,
                    uuid=instance.uuid
                )
            except ValidationError, e:  # pragma: no cover
                self.stderr.write('An error occured with: %s(%s)' % (
                    instance.title, instance.uuid))
                self.stderr.write(e)

        # Manually refresh stuff because the command disables signals
        workspace.refresh_index()

        # second pass to add related fields
        for instance in pages:
            if instance.source:
                p = Post.objects.get(uuid=instance.uuid)
                p.source = Post.objects.get(uuid=instance.source)
                p.save()

            if instance.linked_pages:
                p = Post.objects.get(uuid=instance.uuid)
                p.related_posts.add(*list(
                    Post.objects.filter(uuid__in=instance.linked_pages)))
                p.save()
        self.emit('done.')
        self.reconnect_signals()

    def get_input_data(self, message, default=None):
        raw_value = self.input_func(message)
        if default and raw_value == '':
            raw_value = default

        return raw_value.lower()



from StringIO import StringIO

from django.core.management import call_command

from cms.models import Post, Category
from cms.tests.base import BaseCmsTestCase
from cms.management.commands import import_from_git

from unicore.content import models as eg_models


class TestImportFromGit(BaseCmsTestCase):

    def setUp(self):
        self.workspace = self.mk_workspace()

    def test_command(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            cat1, cat2 = self.create_categories(self.workspace, position=3)
            self.workspace.save(cat1.update({
                'source': cat2.uuid,
                'position': 4,
            }), 'Added source to category.')

            pages = self.create_pages(self.workspace, count=10)
            for page in pages[:8]:
                up = page.update({
                    'primary_category': cat1.uuid,
                })
                self.workspace.save(up, 'Added category.')

            [page0] = self.workspace.S(
                eg_models.Page).filter(uuid=pages[0].uuid)
            original = page0.get_object()
            updated = original.update({
                'linked_pages': [page.uuid for page in pages[:3]],
                'source': pages[4].uuid,
            })
            self.workspace.save(updated, 'Added related fields.')

            self.assertEquals(Category.objects.all().count(), 0)
            self.assertEquals(Post.objects.all().count(), 0)

            call_command('import_from_git', quiet=True)

            self.assertEquals(Category.objects.all().count(), 2)
            self.assertEquals(Post.objects.all().count(), 10)

            c = Category.objects.get(uuid=cat1.uuid)
            self.assertEquals(c.source.uuid, cat2.uuid)
            self.assertEquals(c.position, 4)

            p = Post.objects.get(uuid=page0.uuid)
            self.assertEquals(p.related_posts.count(), 3)
            self.assertEquals(p.primary_category.uuid, cat1.uuid)
            self.assertEquals(p.source.uuid, pages[4].uuid)

    def test_get_input_data(self):

        self.captured_message = None

        def patched_input_func(message):
            self.captured_message = message
            return 'y'

        command = import_from_git.Command()
        command.stdout = StringIO()
        command.input_func = patched_input_func
        command.handle(quiet=False)

        self.assertEquals(
            self.captured_message,
            'Do you want to delete existing data? Y/n: ')
        self.assertEquals(command.stdout.getvalue(), (
            'deleting existing content..'
            'creating categories..'
            'done.'))

    def test_get_input_data_with_default(self):

        self.captured_message = None

        def patched_input_func(message):
            self.captured_message = message
            return ''

        command = import_from_git.Command()
        command.stdout = StringIO()
        command.input_func = patched_input_func
        command.handle(quiet=False)

        self.assertEquals(
            self.captured_message,
            'Do you want to delete existing data? Y/n: ')
        self.assertEquals(command.stdout.getvalue(), (
            'deleting existing content..'
            'creating categories..'
            'done.'))

    def test_get_input_data_with_quiet_set(self):
        command = import_from_git.Command()
        command.stdout = StringIO()
        command.handle(quiet=True)
        self.assertEquals(command.stdout.getvalue(), '')



import json
from datetime import datetime

# ensure celery autodiscovery runs
from djcelery import admin as celery_admin

from djcelery.models import (
    TaskState, WorkerState, PeriodicTask, IntervalSchedule, CrontabSchedule)

from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.conf import settings
from django.contrib.admin.util import unquote
from django.contrib.admin import helpers
from django.http import Http404, HttpResponse
from django.utils.html import escape
from django.core.exceptions import PermissionDenied

from cms.models import (
    Post, Category, Localisation, ContentRepository, PublishingTarget)
from cms.forms import PostForm, CategoryForm
from cms import tasks

from elasticgit import EG


class CategoriesListFilter(SimpleListFilter):
    title = "categories"
    parameter_name = "category_slug"

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return Category.objects.values_list('slug', 'title')

    def queryset(self, request, queryset):
        """
        Returns queryset filtered on categories and primary_category.
        """
        if self.value():
            return queryset.filter(primary_category__slug=self.value())


class PostSourceListFilter(SimpleListFilter):
    title = "sources"
    parameter_name = "source_slug"

    def lookups(self, request, model_admin):
        return Post.objects.filter(
            post__isnull=False).values_list('slug', 'title')

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(source__slug=self.value())


class CategorySourceListFilter(SimpleListFilter):
    title = "sources"
    parameter_name = "source_slug"

    def lookups(self, request, model_admin):
        return Category.objects.filter(
            category__isnull=False).distinct().values_list('slug', 'title')

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(source__slug=self.value())


class TranslatableModelAdmin(admin.ModelAdmin):

    def add_view(self, request, form_url='', extra_context=None):
        object_id = request.GET.get('source', '')
        extra_context = extra_context or {}
        obj = self.get_object(request, unquote(object_id))

        if obj is not None:
            ModelForm = self.get_form(request, obj)
            form = ModelForm(instance=obj)
            extra_context['sourceform'] = helpers.AdminForm(
                form, self.get_fieldsets(request, obj),
                self.get_prepopulated_fields(request, obj),
                self.get_readonly_fields(request, obj),
                model_admin=self)
        return super(TranslatableModelAdmin, self).add_view(
            request, form_url, extra_context)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        extra_context = extra_context or {}
        obj = self.get_object(request, unquote(object_id))

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        if obj is None:
            raise Http404(
                'Post object with primary key %(key)r does not exist.' %
                {'key': escape(object_id)})

        if obj.source:
            ModelForm = self.get_form(request, obj.source)
            form = ModelForm(instance=obj.source)
            extra_context['sourceform'] = helpers.AdminForm(
                form, self.get_fieldsets(request, obj.source),
                self.get_prepopulated_fields(request, obj.source),
                self.get_readonly_fields(request, obj.source),
                model_admin=self)
        return super(TranslatableModelAdmin, self).change_view(
            request, object_id, form_url, extra_context)


class PostAdmin(TranslatableModelAdmin):
    form = PostForm

    list_display = (
        'title', 'subtitle', 'primary_category', 'created_at', 'localisation',
        'source', '_derivatives', 'uuid', 'featured_in_category', 'featured')

    list_filter = (
        'featured_in_category', 'featured', 'created_at', 'localisation',
        CategoriesListFilter, PostSourceListFilter
    )
    prepopulated_fields = {"slug": ("title",)}
    search_fields = ('title', 'description', 'content')
    raw_id_fields = ('owner', 'source')
    fieldsets = (
        (None, {'fields': (
            'title', 'slug', 'subtitle', 'description', 'content', )}),
        (None, {'fields': (
            'primary_category',
            'localisation',
            'featured_in_category',
            'featured',
            'related_posts',
        )}),
        ('Meta', {
            'fields': ('owner', 'created_at', 'source'),
            'classes': ('grp-collapse grp-closed',)})
    )

    def _derivatives(self, post):
        return post.post_set.count()
    _derivatives.short_description = 'Derivatives'
    _derivatives.allow_tags = True

    def save_model(self, request, obj, form, change):
        if not obj.owner:
            obj.owner = request.user
        obj.last_author = request.user

        super(PostAdmin, self).save_model(
            request,
            obj,
            form,
            change
        )


class PostInline(admin.StackedInline):
    model = Post
    extra = 0
    sortable_field_name = 'position'
    sortable_excludes = ('primary_category',)
    raw_id_fields = ('owner', 'source', )
    fields = ('title', 'position')
    readonly_fields = ('title', )


class CategoryAdmin(TranslatableModelAdmin):
    form = CategoryForm

    list_filter = ('localisation', CategorySourceListFilter)
    list_display = (
        'title', 'subtitle', 'localisation', 'featured_in_navbar', 'source',
        '_derivatives', 'uuid',)

    raw_id_fields = ('source', )
    prepopulated_fields = {"slug": ("title",)}
    fieldsets = (
        (None, {'fields': ('title', 'slug', 'subtitle')}),
        (None, {'fields': ('localisation', 'featured_in_navbar',)}),
        ('Meta', {
            'fields': ('source', ),
            'classes': ('grp-collapse grp-closed', )})
    )
    inlines = (PostInline, )

    def _derivatives(self, category):
        return category.category_set.count()
    _derivatives.short_description = 'Derivatives'
    _derivatives.allow_tags = True

    def save_model(self, request, obj, form, change):
        obj.last_author = request.user
        super(CategoryAdmin, self).save_model(request, obj, form, change)


class CategoryInline(admin.StackedInline):
    model = Category
    extra = 0
    sortable_field_name = 'position'
    sortable_excludes = ('localisation',)
    raw_id_fields = ('source', 'last_author')
    fields = ('title', 'position')
    readonly_fields = ('title', )


class LocalisationAdmin(admin.ModelAdmin):
    inlines = (CategoryInline,)


class ContentRepositoryAdmin(admin.ModelAdmin):

    readonly_fields = ('url', 'name', 'targets')

    def get_object(self, request, object_id):
        obj = super(ContentRepositoryAdmin, self).get_object(
            request, object_id)
        if obj is None:  # pragma: no cover
            return

        if not obj.targets.exists():
            obj.targets.add(PublishingTarget.get_default_target())
        return obj

    def has_add_permission(self, *args, **kwargs):
        return not ContentRepository.objects.exists()


class PublishingTargetAdmin(admin.ModelAdmin):
    readonly_fields = ('url', 'name')

    def has_add_permission(self, *args, **kwargs):
        PublishingTarget.get_default_target()
        return False


@admin.site.register_view('github/', 'Github Configuration')
def my_view(request, *args, **kwargs):
    workspace = EG.workspace(settings.GIT_REPO_PATH,
                             index_prefix=settings.ELASTIC_GIT_INDEX_PREFIX)
    commits = workspace.repo.iter_commits(max_count=10)

    context = {
        'github_url': settings.GIT_REPO_URL,
        'repo': workspace.repo,
        'commits': [
            {
                'message': c.message,
                'author': c.author.name,
                'commit_time': datetime.fromtimestamp(c.committed_date)
            }
            for c in commits
        ]
    }
    return render(request, 'cms/admin/github.html', context)


@admin.site.register_view('github/push/', 'Push to github')
def push_to_github(request, *args, **kwargs):
    tasks.push_to_git.delay(settings.GIT_REPO_PATH,
                            settings.ELASTIC_GIT_INDEX_PREFIX)
    if request.is_ajax():
        return HttpResponse(
            json.dumps({'success': True}),
            mimetype='application/json')
    return redirect(reverse('admin:index'))


admin.site.register(Post, PostAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(ContentRepository, ContentRepositoryAdmin)
admin.site.register(PublishingTarget, PublishingTargetAdmin)

# remove celery from admin
admin.site.unregister(TaskState)
admin.site.unregister(WorkerState)
admin.site.unregister(IntervalSchedule)
admin.site.unregister(CrontabSchedule)
admin.site.unregister(PeriodicTask)

admin.site.register(Localisation, LocalisationAdmin)



from cms.tasks import push_to_git
from cms.tests.base import BaseCmsTestCase
from django.conf import settings

from unicore.content.models import Page


class TaskTest(BaseCmsTestCase):

    def setUp(self):
        self.local_workspace = self.mk_workspace()
        self.remote_workspace = self.mk_workspace(
            name='%s_remote' % (self.id().lower(),),
            index_prefix='%s_remote' % (self.local_workspace.index_prefix,))

        for data, count in self.create_page_data_iter():
            data['title'] = 'remote page %i' % (count,)
            self.remote_workspace.save(
                Page(data), 'Saving %(title)s.' % data)
        self.remote_workspace.refresh_index()

    def test_push_to_git(self):
        # NOTE: In order to push to the remote workspace's master branch it
        #       needs to itself be checked out with a different branch,
        #       one cannot pushed to branches that are currently checked out.
        remote_repo = self.remote_workspace.repo
        remote_repo.git.checkout('HEAD', b='temp')

        with self.active_workspace(self.local_workspace):
            origin = self.local_workspace.repo.create_remote(
                'origin', self.remote_workspace.working_dir)
            branch = self.local_workspace.repo.active_branch
            origin.fetch()
            remote_master = origin.refs.master
            branch.set_tracking_branch(remote_master)

            self.local_workspace.fast_forward()
            self.local_workspace.reindex(Page)

            self.assertEqual(self.local_workspace.S(Page).count(), 2)
            self.create_pages(self.local_workspace)
            self.assertEqual(self.local_workspace.S(Page).count(), 4)

            push_to_git(settings.GIT_REPO_PATH,
                        settings.ELASTIC_GIT_INDEX_PREFIX)

        # NOTE: switching back to master on the remote because the changes
        #       should have been pushed and we are checking for their
        #       existence

        remote_repo.heads.master.checkout()

        self.assertEqual(self.remote_workspace.S(Page).count(), 2)
        self.remote_workspace.reindex(Page)
        self.assertEqual(self.remote_workspace.S(Page).count(), 4)



from pagedown.widgets import AdminPagedownWidget
from django import forms
from cms.models import Post, Category


class PostForm(forms.ModelForm):
    content = forms.CharField(widget=AdminPagedownWidget())

    def __init__(self, *args, **kwargs):
        super(PostForm, self).__init__(*args, **kwargs)
        self.fields['localisation'].required = True

    class Meta:
        model = Post


class CategoryForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(CategoryForm, self).__init__(*args, **kwargs)
        self.fields['localisation'].required = True

    class Meta:
        model = Category



from cms.tests.base import BaseCmsTestCase

from cms.forms import PostForm, CategoryForm


class FormTest(BaseCmsTestCase):

    def test_required_localisations(self):
        post_form = PostForm()
        self.assertTrue(post_form.fields['localisation'].required)
        category_form = CategoryForm()
        self.assertTrue(category_form.fields['localisation'].required)



import os
from uuid import uuid4

from django.test import TestCase
from django.conf import settings

from elasticgit import EG

from slugify import slugify

from unicore.content.models import Page, Category


class BaseCmsTestCase(TestCase):

    destroy = 'KEEP_REPO' not in os.environ

    def mk_workspace(self, working_dir='.test_repos/',
                     name=None,
                     url='https://localhost',
                     index_prefix=None,
                     auto_destroy=None,
                     author_name='Test Kees',
                     author_email='kees@example.org'):
        name = name or self.id()
        index_prefix = (
            index_prefix or
            settings.ELASTIC_GIT_INDEX_PREFIX or
            name.lower().replace('.', '-'))
        auto_destroy = auto_destroy or self.destroy
        workspace = EG.workspace(os.path.join(working_dir, name), es={
            'urls': [url],
        }, index_prefix=index_prefix)
        if auto_destroy:
            self.addCleanup(workspace.destroy)

        workspace.setup(author_name, author_email)
        while not workspace.index_ready():
            pass

        return workspace

    def create_category_data_iter(self, count=2, locale='eng_UK', **kwargs):
        for i in range(count):
            data = {}
            data.update({
                'title': u'Test Category %s' % (i,),
                'language': locale
            })
            data.update(kwargs)
            data.update({
                'slug': slugify(data['title'])
            })
            yield data, i

    def create_categories(
            self, workspace, count=2, locale='eng_UK', **kwargs):
        categories = []
        for data, i in self.create_category_data_iter(
                count=2, locale=locale, **kwargs):
            category = Category(data)
            workspace.save(
                category, u'Added category %s.' % (i,))
            categories.append(category)

        workspace.refresh_index()
        return categories

    def create_page_data_iter(self, count=2, locale='eng_UK', **kwargs):
        for i in range(count):
            data = {}
            data.update({
                'title': u'Test Page %s' % (i,),
                'content': u'this is sample content for pg %s' % (i,),
                'language': locale
            })
            data.update(kwargs)
            data.update({
                'slug': slugify(data['title'])
            })
            yield data, i

    def create_pages(
            self, workspace, count=2, locale='eng_UK', **kwargs):
        pages = []
        for data, i in self.create_page_data_iter(
                count=count, locale=locale, **kwargs):
            page = Page(data)
            workspace.save(page, message=u'Added page %s.' % (i,))
            pages.append(page)

        workspace.refresh_index()
        return pages

    def active_workspace(self, workspace):
        return self.settings(
            GIT_REPO_PATH=workspace.working_dir,
            ELASTIC_GIT_INDEX_PREFIX=workspace.index_prefix)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Category'
        db.create_table(u'cms_category', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('uuid', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=32, unique=True, null=True, blank=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('subtitle', self.gf('django.db.models.fields.CharField')(default='', max_length=200, null=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=255)),
        ))
        db.send_create_signal(u'cms', ['Category'])

        # Adding model 'Post'
        db.create_table(u'cms_post', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('uuid', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=32, unique=True, null=True, blank=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('subtitle', self.gf('django.db.models.fields.CharField')(default='', max_length=200, null=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=255)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(db_index=True, blank=True)),
            ('modified_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, db_index=True, blank=True)),
            ('owner', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True, blank=True)),
            ('primary_category', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='primary_modelbase_set', null=True, to=orm['cms.Category'])),
        ))
        db.send_create_signal(u'cms', ['Post'])

        # Adding M2M table for field categories on 'Post'
        m2m_table_name = db.shorten_name(u'cms_post_categories')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('post', models.ForeignKey(orm[u'cms.post'], null=False)),
            ('category', models.ForeignKey(orm[u'cms.category'], null=False))
        ))
        db.create_unique(m2m_table_name, ['post_id', 'category_id'])


    def backwards(self, orm):
        # Deleting model 'Category'
        db.delete_table(u'cms_category')

        # Deleting model 'Post'
        db.delete_table(u'cms_post')

        # Removing M2M table for field categories on 'Post'
        db.delete_table(db.shorten_name(u'cms_post_categories'))


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'cms.category': {
            'Meta': {'ordering': "('title',)", 'object_name': 'Category'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'subtitle': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'uuid': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '32', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        u'cms.post': {
            'Meta': {'ordering': "('-created_at',)", 'object_name': 'Post'},
            'categories': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': u"orm['cms.Category']", 'null': 'True', 'blank': 'True'}),
            'content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'db_index': 'True', 'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'primary_category': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'primary_modelbase_set'", 'null': 'True', 'to': u"orm['cms.Category']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'subtitle': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'uuid': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '32', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['cms']


from django.conf.urls import patterns, include, url
from django.contrib import admin
from django.views.generic.base import RedirectView

from adminplus.sites import AdminSitePlus

admin.site = AdminSitePlus()
admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^admin/', RedirectView.as_view(url='/')),
    url(r'^grappelli/', include('grappelli.urls')),
    url(r'^', include(admin.site.urls)),
)



import os

from django import forms
from django.conf import settings
from django.contrib import admin
from django.test.utils import override_settings

from cms.tests.base import BaseCmsTestCase
from cms.models import (
    ContentRepository, PublishingTarget, CUSTOM_REPO_LICENSE_TYPE)
from cms.admin import ContentRepositoryAdmin


@override_settings(GIT_REPO_URL='git@host.com/foo.git',
                   DEFAULT_TARGET_NAME='The Target')
class TestContentRepository(BaseCmsTestCase):

    def setUp(self):
        self.workspace = self.mk_workspace()

    def test_default_license(self):
        repo = ContentRepository.objects.create()
        self.assertTrue(repo.get_license_text())

    def test_get_license(self):
        repo = ContentRepository.objects.create(license='CC-BY-4.0')
        text = repo.get_license_text().strip()
        self.assertTrue(
            text.startswith('Attribution 4.0 International'))
        self.assertTrue(
            text.endswith(
                'Creative Commons may be contacted at creativecommons.org.'))

    def test_write_license_file(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            repo = ContentRepository(license='CC-BY-4.0')
            repo.save()
            file_path = os.path.join(settings.GIT_REPO_PATH, 'LICENSE')
            with open(file_path, 'r') as fp:
                license_text = fp.read()
            self.assertEqual(license_text, repo.get_license_text())

    def test_custom_license_text(self):
        repo = ContentRepository(
            license=CUSTOM_REPO_LICENSE_TYPE,
            custom_license_name='Foo',
            custom_license_text='Bar')
        repo.save()
        self.assertEqual(repo.get_license_text(), 'Bar')
        self.assertEqual(unicode(repo), 'Foo (Custom license)')

    def test_validation(self):
        self.assertRaises(
            forms.ValidationError,
            ContentRepository(license=CUSTOM_REPO_LICENSE_TYPE).full_clean)
        self.assertRaises(
            forms.ValidationError,
            ContentRepository(
                license=CUSTOM_REPO_LICENSE_TYPE,
                custom_license_name='Foo').full_clean)
        self.assertRaises(
            forms.ValidationError,
            ContentRepository(
                license=CUSTOM_REPO_LICENSE_TYPE,
                custom_license_text='Bar').full_clean)

    def test_setting_of_target_fields(self):
        cr = ContentRepository(license='CC-BY-4.0')
        cr.save()
        self.assertEqual(cr.url, 'git@host.com/foo.git')
        self.assertEqual(cr.name, 'foo')
        self.assertFalse(cr.targets.exists())
        self.assertEqual(PublishingTarget.objects.count(), 0)

        model_admin = ContentRepositoryAdmin(ContentRepository, admin)
        obj = model_admin.get_object(None, cr.pk)
        self.assertEqual(obj.url, 'git@host.com/foo.git')
        self.assertEqual(obj.name, 'foo')
        self.assertEqual(PublishingTarget.objects.count(), 1)
        [target] = obj.targets.all()
        self.assertEqual(target.name, 'The Target')


class TestPublishingTarget(BaseCmsTestCase):

    def test_get_default_target(self):
        self.assertFalse(PublishingTarget.objects.exists())
        default_target = PublishingTarget.get_default_target()
        self.assertEqual(default_target.name, settings.DEFAULT_TARGET_NAME)
        self.assertTrue(PublishingTarget.objects.exists())



from django.conf import settings
from elasticgit import EG

from cms.models import ContentRepository


def workspace_changes(request):
    workspace = EG.workspace(settings.GIT_REPO_PATH,
                             index_prefix=settings.ELASTIC_GIT_INDEX_PREFIX)
    repo = workspace.repo
    index = repo.index
    origin = repo.remote()
    [fetch_info] = origin.fetch()
    remote_master = origin.refs.master
    return {
        'repo_changes': len(index.diff(remote_master.commit)),
    }


def content_repositories(request):
    return {
        'content_repositories': ContentRepository.objects.all(),
    }



# Django settings for skeleton project.

import os
import djcelery

djcelery.setup_loader()

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))


def abspath(*args):
    """convert relative paths to absolute paths relative to PROJECT_ROOT"""
    return os.path.join(PROJECT_ROOT, *args)

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'cms_django.db',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts
ALLOWED_HOSTS = ['*']

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'UTC'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"
MEDIA_ROOT = abspath('media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = abspath('static')

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'django.contrib.staticfiles.finders.FileSystemFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
# Leaving this intentionally blank because you have to generate one yourself.
SECRET_KEY = 'please-change-me'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.core.context_processors.tz",
    "django.contrib.messages.context_processors.messages",
    "django.core.context_processors.request",
    "cms.context_processors.workspace_changes",
    "cms.context_processors.content_repositories",
)

ROOT_URLCONF = 'project.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'project.wsgi.application'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    abspath('templates'),
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Uncomment the next line to enable the admin:
    'grappelli',
    'django.contrib.admin',
    # Uncomment the next line to enable admin documentation:
    # 'django.contrib.admindocs',
    'south',
    'gunicorn',
    'django_nose',
    'raven.contrib.django.raven_compat',
    'djcelery',
    'debug_toolbar',
    'sortedm2m',
    'pagedown',

    # sample apps to explain usage
    'cms',
    'adminplus',
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

# Celery configuration options
BROKER_URL = 'redis://localhost:6379/0'
CELERY_RESULT_BACKEND = 'redis://localhost:6379/0'

# Uncomment if you're running in DEBUG mode and you want to skip the broker
# and execute tasks immediate instead of deferring them to the queue / workers.
CELERY_ALWAYS_EAGER = DEBUG

# Tell Celery where to find the tasks
CELERY_IMPORTS = ('cms.tasks',)

# Defer email sending to Celery, except if we're in debug mode,
# then just print the emails to stdout for debugging.
EMAIL_BACKEND = 'djcelery_email.backends.CeleryEmailBackend'
if DEBUG:
    EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

# Django debug toolbar
DEBUG_TOOLBAR_CONFIG = {
    'INTERCEPT_REDIRECTS': False,
    'ENABLE_STACKTRACES': True,
}

# South configuration variables
TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
SKIP_SOUTH_TESTS = True     # Do not run the south tests as part of our
                            # test suite.
SOUTH_TESTS_MIGRATE = False  # Do not run the migrations for our tests.
                             # We are assuming that our models.py are correct
                             # for the tests and as such nothing needs to be
                             # migrated.

# Sentry configuration
RAVEN_CONFIG = {
    # DevOps will supply you with this.
    # 'dsn': 'http://public:secret@example.com/1',
}

CKEDITOR_UPLOAD_PATH = abspath('media', 'uploads')
CKEDITOR_CONFIGS = {
    'default': {
        'toolbar': [
        ['Styles', 'Format', 'Bold', 'Italic', 'Underline',
            'Strike', 'SpellChecker', 'Undo', 'Redo'],
        ['TextColor', 'BGColor'],
        ['NumberedList', 'BulletedList'],
        ['Smiley', 'SpecialChar'], ['Source'],
        ],
        'forcePasteAsPlainText': True,
    }
}

GRAPPELLI_ADMIN_TITLE = 'Universal Core'

GIT_REPO_URL = None
GIT_REPO_PATH = abspath('cmsrepo')
DEFAULT_TARGET_NAME = 'Default Target'
ELASTIC_GIT_INDEX_PREFIX = 'elastic-git'

# used when pushing to Github
SSH_PUBKEY_PATH = None
SSH_PRIVKEY_PATH = None
SSH_PASSPHRASE = ''

try:
    from local_settings import *
except ImportError:
    pass



from cms.tests.base import BaseCmsTestCase
from django.test.client import RequestFactory

from cms import context_processors


class TestContextProcessors(BaseCmsTestCase):

    def test_workspace_changes(self):
        local_workspace = self.mk_workspace()
        remote_workspace = self.mk_workspace(
            name='%s_remote' % (self.id().lower(),))

        # the local grows some categories
        self.create_categories(remote_workspace)

        local_repo = local_workspace.repo
        origin = local_repo.create_remote(
            'origin', remote_workspace.working_dir)
        [fetch_info] = origin.fetch()
        local_repo.git.merge(fetch_info.commit)

        # Make 2 local changes
        self.create_pages(local_workspace)

        request = RequestFactory().get('/')
        with self.active_workspace(local_workspace):
            context = context_processors.workspace_changes(request)
            self.assertEqual(context, {
                'repo_changes': 2,
            })

    def test_content_repositories(self):
        request = RequestFactory().get('/')
        context = context_processors.content_repositories(request)
        self.assertEqual(list(context['content_repositories']), [])



from cms.models import Post, Category, Localisation
from cms.tests.base import BaseCmsTestCase

from unicore.content import models as eg_models


class PostTestCase(BaseCmsTestCase):

    def setUp(self):
        self.workspace = self.mk_workspace()

    def test_create_post(self):
        p = Post(
            title='sample title',
            description='description',
            subtitle='subtitle',
            content='sample content',
            position=3)
        p.save()
        self.assertEquals(p.featured_in_category, False)
        self.assertEquals(Post.objects.all().count(), 1)
        self.assertEquals(self.workspace.S(eg_models.Page).count(), 1)

        p = Post.objects.get(pk=p.pk)
        p.title = 'changed title'
        p.save()

        self.assertEquals(self.workspace.S(eg_models.Page).count(), 1)
        [eg_page] = self.workspace.S(eg_models.Page).everything()
        self.assertEquals(eg_page.title, 'changed title')
        self.assertEquals(eg_page.uuid, p.uuid)
        self.assertEquals(eg_page.subtitle, 'subtitle')
        self.assertEquals(eg_page.description, 'description')
        self.assertEquals(eg_page.featured_in_category, False)
        self.assertEquals(eg_page.position, 3)
        self.assertTrue(eg_page.created_at is not None)
        self.assertTrue(eg_page.modified_at is not None)

        p.delete()
        self.assertEquals(Post.objects.all().count(), 0)
        self.assertEquals(self.workspace.S(eg_models.Page).count(), 0)

    def test_create_category(self):
        c = Category(
            title='sample title',
            subtitle='subtitle',
            slug='sample-title')
        c.save()
        self.assertEquals(Category.objects.all().count(), 1)
        self.assertEquals(self.workspace.S(eg_models.Category).count(), 1)

        c = Category.objects.get(pk=c.pk)
        c.title = 'changed title'
        c.save()

        self.assertEquals(self.workspace.S(eg_models.Category).count(), 1)
        [git_cat] = self.workspace.S(eg_models.Category).everything()
        self.assertEquals(git_cat.title, 'changed title')
        self.assertEquals(git_cat.uuid, c.uuid)
        self.assertEquals(git_cat.subtitle, 'subtitle')

        c.delete()
        self.assertEquals(Category.objects.all().count(), 0)
        self.assertEquals(self.workspace.S(eg_models.Category).count(), 0)

    def test_page_with_primary_category(self):
        c = Category(
            title='guides',
            slug='guides')
        c.save()
        c = Category.objects.get(pk=c.pk)

        p = Post(
            title='sample title',
            description='description',
            subtitle='subtitle',
            content='sample content')
        p.primary_category = c
        p.save()

        p = Post.objects.get(pk=p.pk)

        [git_p] = self.workspace.S(eg_models.Page).filter(uuid=p.uuid)
        self.assertEquals(git_p.primary_category, c.uuid)

    def test_page_recreated_if_not_in_git(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            p = Post(
                title='sample test title',
                description='description',
                subtitle='subtitle',
                content='sample content')
            p.save()
            p = Post.objects.get(pk=p.pk)

            # Manually delete the git page
            [git_page] = self.workspace.S(eg_models.Page).filter(uuid=p.uuid)
            self.workspace.delete(git_page.get_object(),
                                  'Removing: %s' % p.uuid)
            self.workspace.refresh_index()
            self.assertEquals(
                self.workspace.S(eg_models.Page).filter(uuid=p.uuid).count(),
                0)

            p.title = 'new title'
            p.save()

            p = Post.objects.get(pk=p.pk)
            [git_p] = self.workspace.S(eg_models.Page).filter(uuid=p.uuid)

            self.assertEquals(git_p.title, 'new title')

    def test_category_recreated_if_not_in_git(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            c = Category(
                title='sample test title',
                slug='slug')
            c.save()
            c = Category.objects.get(pk=c.pk)
            [git_c] = self.workspace.S(eg_models.Category).filter(uuid=c.uuid)
            self.workspace.delete(git_c.get_object(),
                                  'Deleting: %s' % (c.uuid,))
            c.title = 'new title'
            c.save()

            c = Category.objects.get(pk=c.pk)
            [git_c] = self.workspace.S(eg_models.Category).filter(uuid=c.uuid)

            self.assertEquals(git_c.title, 'new title')

    def test_page_with_source(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            c = Category(
                title='guides',
                slug='guides')
            c.save()
            c = Category.objects.get(pk=c.pk)

            p = Post(
                title='sample title',
                description='description',
                subtitle='subtitle',
                content='sample content',
                localisation=Localisation._for('eng_UK'))
            p.save()
            p = Post.objects.get(pk=p.pk)

            p2 = Post(
                title='sample title',
                description='description',
                subtitle='subtitle',
                content='sample content',
                localisation=Localisation._for('eng_US'))
            p2.primary_category = c
            p2.source = p
            p2.save()
            p2 = Post.objects.get(pk=p2.pk)

            [git_p2] = self.workspace.S(eg_models.Page).filter(uuid=p2.uuid)
            [git_p2_source] = self.workspace.S(eg_models.Page).filter(
                uuid=p2.source.uuid)
            self.assertEquals(git_p2.language, 'eng_US')
            self.assertEquals(git_p2_source.language, 'eng_UK')

            p2.source = None
            p2.primary_category = None
            p2.save()

            [git_p2] = self.workspace.S(eg_models.Page).filter(uuid=p2.uuid)
            self.assertEquals(git_p2.source, None)
            self.assertEquals(git_p2.primary_category, None)

    def test_page_featured_in_category(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            p = Post(
                title='sample title',
                description='description',
                subtitle='subtitle',
                content='sample content',
                localisation=Localisation._for('eng_UK'),
                featured_in_category=True)
            p.save()

            p = Post.objects.get(pk=p.pk)
            [git_p] = self.workspace.S(eg_models.Page).filter(uuid=p.uuid)
            self.assertTrue(git_p.featured_in_category)

    def test_page_get_featured(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            post = Post(
                title='sample title',
                description='description',
                subtitle='subtitle',
                content='sample content',
                localisation=Localisation._for('eng_UK'))
            post.save()

            featured_post = Post(
                title='featured sample title',
                description='featured description',
                subtitle='featured subtitle',
                content='featured sample content',
                localisation=Localisation._for('eng_UK'),
                featured=True)
            featured_post.save()

            post = Post.objects.get(pk=post.pk)
            [git_post] = self.workspace.S(
                eg_models.Page).filter(uuid=post.uuid)

            featured_post = Post.objects.get(pk=featured_post.pk)
            [featured_git_post] = self.workspace.S(
                eg_models.Page).filter(uuid=featured_post.uuid)

            self.assertEqual(post.featured, False)
            self.assertEquals(git_post.featured, False)

            self.assertEqual(featured_post.featured, True)
            self.assertEquals(featured_git_post.featured, True)

    def test_category_with_source(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            c = Category(
                title='sample title',
                subtitle='subtitle',
                localisation=Localisation._for('afr_ZA'))
            c.save()
            c2 = Category(
                title='sample title',
                subtitle='subtitle',
                localisation=Localisation._for('eng_UK'))
            c2.save()

            c = Category.objects.get(pk=c.pk)
            c2 = Category.objects.get(pk=c2.pk)
            c2.source = c
            c2.save()

            [git_c2] = self.workspace.S(
                eg_models.Category).filter(uuid=c2.uuid)
            self.assertEquals(git_c2.language, 'eng_UK')
            [source] = self.workspace.S(
                eg_models.Category).filter(uuid=git_c2.source)
            self.assertEquals(source.language, 'afr_ZA')

            c2.source = None
            c2.save()

            [git_c2] = self.workspace.S(
                eg_models.Category).filter(uuid=c2.uuid)
            self.assertEquals(git_c2.source, None)

    def test_category_with_featured_in_navbar(self):
        with self.settings(GIT_REPO_PATH=self.workspace.working_dir):
            c = Category(
                title='sample title',
                subtitle='subtitle',
                localisation=Localisation._for('afr_ZA'),
                featured_in_navbar=True)
            c.save()

            c = Category.objects.get(pk=c.pk)
            [git_c] = self.workspace.S(eg_models.Category).filter(uuid=c.uuid)
            self.assertTrue(git_c.featured_in_navbar)

    def test_localisation_for_helper(self):
        localisations = Localisation.objects.filter(
            language_code='eng', country_code='UK')
        self.assertEqual(localisations.count(), 0)
        localisation1 = Localisation._for('eng_UK')
        localisation2 = Localisation._for('eng_UK')
        self.assertEqual(localisations.count(), 1)
        self.assertEquals(localisation1.pk, localisation2.pk)

    def test_localisation_get_code_helper(self):
        self.assertEqual(
            Localisation._for('eng_UK').get_code(),
            'eng_UK')

    def test_category_position_is_saved(self):
        c = Category(
            title='sample title',
            subtitle='subtitle',
            localisation=Localisation._for('afr_ZA'),
            featured_in_navbar=True,
            position=4)
        c.save()

        c = Category.objects.get(pk=c.pk)
        [git_c] = self.workspace.S(eg_models.Category).filter(uuid=c.uuid)
        self.assertEquals(git_c.position, 4)

    def test_page_ordering(self):
        Post.objects.create(
            title=u'New page',
            content=u'New page sample content',
            localisation=Localisation._for('afr_ZA'),
        )
        self.assertEquals(Post.objects.all()[0].title, 'New page')
        self.assertEquals(Post.objects.all()[0].position, 0)

        Post.objects.create(
            title=u'New page 2',
            content=u'New page sample content 2',
            localisation=Localisation._for('afr_ZA'),
        )
        self.assertEquals(Post.objects.all()[0].title, 'New page 2')
        self.assertEquals(Post.objects.all()[0].position, 0)
        self.assertEquals(Post.objects.all()[1].title, 'New page')
        self.assertEquals(Post.objects.all()[1].position, 1)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Post.position'
        db.add_column(u'cms_post', 'position',
                      self.gf('django.db.models.fields.PositiveIntegerField')(default=0, null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Post.position'
        db.delete_column(u'cms_post', 'position')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'cms.category': {
            'Meta': {'ordering': "('position', 'title')", 'object_name': 'Category'},
            'featured_in_navbar': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_author': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'category_last_author'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'localisation': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['cms.Localisation']", 'null': 'True', 'blank': 'True'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['cms.Category']", 'null': 'True', 'blank': 'True'}),
            'subtitle': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'uuid': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '32', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        u'cms.localisation': {
            'Meta': {'object_name': 'Localisation'},
            'country_code': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '3'})
        },
        u'cms.post': {
            'Meta': {'ordering': "('-created_at',)", 'object_name': 'Post'},
            'content': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'featured': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'featured_in_category': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_author': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'post_last_author'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'localisation': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['cms.Localisation']", 'null': 'True', 'blank': 'True'}),
            'modified_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'db_index': 'True', 'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'primary_category': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'primary_modelbase_set'", 'null': 'True', 'to': u"orm['cms.Category']"}),
            'related_posts': ('sortedm2m.fields.SortedManyToManyField', [], {'related_name': "'related_posts_rel_+'", 'blank': 'True', 'to': u"orm['cms.Post']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'source': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['cms.Post']", 'null': 'True', 'blank': 'True'}),
            'subtitle': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'uuid': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '32', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['cms']


import os

from django.contrib.auth.models import User
from django.conf import settings
from django.db import models
from django.db.models import F
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from django.template.defaultfilters import slugify
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from sortedm2m.fields import SortedManyToManyField

from elasticgit import EG

from git import GitCommandError

from unicore.content import models as eg_models


ENG_UK = 'eng_UK'
SWH_TZ = 'swh_TZ'  # Swahili
SWH_KE = 'swh_KE'  # Swahili
THA_TH = 'tha_TH'  # Thai
IND_ID = 'ind_ID'  # Bahasa

LANGUAGE_CHOICES = (
    (ENG_UK, 'English (United Kingdom)'),
    (SWH_TZ, 'Swahili (Tanzania)'),
    (SWH_KE, 'Swahili (Kenya)'),
    (THA_TH, 'Thai (Thailand)'),
    (IND_ID, 'Bahasa (Indonesia)')
)


CONTENT_REPO_LICENSE_PATH = os.path.join(
    settings.PROJECT_ROOT, '..', 'licenses')

CUSTOM_REPO_LICENSE_TYPE = '_custom'
CONTENT_REPO_LICENSES = (
    (CUSTOM_REPO_LICENSE_TYPE, 'Custom license'),
    ('CC-BY-4.0',
        'Creative Commons Attribution 4.0 International License'),
    ('CC-BY-NC-4.0',
        'Creative Commons Attribution-NonCommercial 4.0 '
        'International License'),
    ('CC-BY-NC-ND-4.0',
        'Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 '
        'International License'),
    ('CC-BY-NC-SA-4.0',
        'Creative Commons Attribution-NonCommercial-ShareAlike 4.0 '
        'International License'),
    ('CC-BY-ND-4.0',
        'Creative Commons Attribution-NoDerivatives 4.0 '
        'International License'),
    ('CC-BY-SA-4.0',
        'Creative Commons Attribution-ShareAlike 4.0 '
        'International License'),
)

DEFAULT_REPO_LICENSE = 'CC-BY-NC-ND-4.0'


class ContentRepository(models.Model):
    # NOTE:
    #       Support for this will grow with tickets focussing on
    #       the targets.
    #
    # This is the entity that'll grow support for what we've been
    # calling targets, it'll store things like:
    #
    # - ssh_public_key
    # - ssh_private_key
    # - ssh_passphrase

    class Meta:
        verbose_name = 'Content Repository Information'
        verbose_name_plural = 'Content Repositories Information'

    @classmethod
    def get_default_name(cls):
        url_head, url_path = settings.GIT_REPO_URL.rsplit('/', 1)
        repo_name, _, dot_git = url_path.rpartition('.')
        return repo_name

    @classmethod
    def get_default_url(cls):
        return settings.GIT_REPO_URL

    name = models.CharField(
        _('The name of the content repository'),
        max_length=255, blank=True, null=True,
        default=lambda: ContentRepository.get_default_name())
    url = models.CharField(
        _('Internet address of where this content repository lives'),
        max_length=255, blank=True, null=True,
        default=lambda: ContentRepository.get_default_url())
    license = models.CharField(
        _('The license to use with this content.'),
        max_length=255, choices=CONTENT_REPO_LICENSES,
        default=DEFAULT_REPO_LICENSE,
        blank=True, null=True)
    custom_license_name = models.CharField(
        _('Name for the custom license.'),
        help_text=_('This name is for your own reference only.'),
        max_length=255, blank=True, null=True)
    custom_license_text = models.TextField(
        _('Should you decide to go with a custom license, paste '
          'the text here.'), blank=True, null=True)
    targets = models.ManyToManyField('PublishingTarget')

    def get_license_text(self):
        if self.license != CUSTOM_REPO_LICENSE_TYPE:
            return self.get_existing_license_text()
        return self.get_custom_license_text()

    def get_custom_license_text(self):
        return self.custom_license_text

    def get_existing_license_text(self):
        file_path = os.path.join(
            CONTENT_REPO_LICENSE_PATH, '%s.txt' % (self.license,))
        with open(file_path) as fp:
            return fp.read()

    def __unicode__(self):  # pragma: no cover
        if self.license == CUSTOM_REPO_LICENSE_TYPE:
            return '%s (%s)' % (self.custom_license_name,
                                self.get_license_display())
        return self.get_license_display()

    def clean(self):
        if (self.license == CUSTOM_REPO_LICENSE_TYPE
                and not all([self.custom_license_text,
                             self.custom_license_name])):
            raise ValidationError(
                'You must specify a license name & text for a custom license.')


class PublishingTarget(models.Model):

    name = models.CharField(
        _('Name for the publishing target.'), max_length=255)
    url = models.URLField(
        _('The Internet address for this target.'))

    @classmethod
    def get_default_target(cls):
        target, _ = cls.objects.get_or_create(
            name=settings.DEFAULT_TARGET_NAME)
        return target

    def __unicode__(self):  # pragma: no cover
        return self.name


class Localisation(models.Model):
    country_code = models.CharField(
        _('2 letter country code'), max_length=2,
        help_text=(
            'See http://www.worldatlas.com/aatlas/ctycodes.htm '
            'for reference.'))
    language_code = models.CharField(
        _('3 letter language code'), max_length=3,
        help_text=(
            'See http://www.loc.gov/standards/iso639-2/php/code_list.php '
            'for reference.'))

    @classmethod
    def _for(cls, language):
        language_code, _, country_code = language.partition('_')
        localisation, _ = cls.objects.get_or_create(
            language_code=language_code, country_code=country_code)
        return localisation

    def get_code(self):
        return u'%s_%s' % (self.language_code, self.country_code)

    def __unicode__(self):  # pragma: no cover
        """
        FIXME: this is probably a bad idea
        """
        language = self.get_code()
        return unicode(dict(LANGUAGE_CHOICES).get(language, language))


class Category(models.Model):
    """
    Category model to be used for categorization of content. Categories are
    high level constructs to be used for grouping and organizing content,
    thus creating a site's table of contents.
    """
    uuid = models.CharField(
        max_length=32,
        blank=True,
        null=True,
        unique=True,
        db_index=True,
        editable=False)
    title = models.CharField(
        max_length=200,
        help_text='Short descriptive name for this category.',
    )
    subtitle = models.CharField(
        max_length=200,
        blank=True,
        null=True,
        default='',
        help_text='Some titles may be the same and cause confusion in admin '
                  'UI. A subtitle makes a distinction.',
    )
    slug = models.SlugField(
        max_length=255,
        db_index=True,
        help_text='Short descriptive unique name for use in urls.',
    )
    last_author = models.ForeignKey(
        User,
        blank=True,
        null=True,
        related_name='category_last_author'
    )
    localisation = models.ForeignKey(
        Localisation, blank=True, null=True)
    source = models.ForeignKey(
        'self',
        blank=True,
        null=True)
    featured_in_navbar = models.BooleanField(
        _('Featured in navigation bar'),
        default=False,
        help_text=_(
            'If checked this category will be displayed on the top'
            'navigation bar. It will always appear on the homepage.'))
    position = models.PositiveIntegerField(
        _('Position in Ordering'), null=True)

    class Meta:
        ordering = ('position', 'title',)
        verbose_name = 'category'
        verbose_name_plural = 'categories'

    def __unicode__(self):  # pragma: no cover
        if self.localisation:
            return '%s (%s)' % (self.title, self.localisation)
        else:
            return self.title

    def save(self, *args, **kwargs):
        # use django slugify filter to slugify
        if not self.slug:
            self.slug = slugify(self.title)

        super(Category, self).save(*args, **kwargs)


class Post(models.Model):

    class Meta:
        ordering = ('position', '-created_at',)

    uuid = models.CharField(
        max_length=32,
        blank=True,
        null=True,
        unique=True,
        db_index=True,
        editable=False)
    title = models.CharField(
        _("Title"),
        max_length=200, help_text=_('A short descriptive title.'),
    )
    subtitle = models.CharField(
        max_length=200,
        blank=True,
        null=True,
        default='',
        help_text=_(
            'Some titles may be the same and cause confusion in admin UI. '
            'A subtitle makes a distinction.'),
    )
    slug = models.SlugField(
        max_length=255,
        db_index=True,
        help_text='Short descriptive unique name for use in urls.',
    )
    description = models.TextField(
        help_text=_(
            'A short description. More verbose than the title but'
            'limited to one or two sentences.'),
        blank=True,
        null=True,
    )
    content = models.TextField(blank=True, null=True)
    created_at = models.DateTimeField(
        _('Created Date & Time'),
        blank=True,
        db_index=True,
        help_text=_(
            'Date and time on which this item was created. This is'
            'automatically set on creation, but can be changed subsequently.')
    )
    modified_at = models.DateTimeField(
        _('Modified Date & Time'),
        db_index=True,
        editable=False,
        auto_now=True,
        help_text=_(
            'Date and time on which this item was last modified. This'
            'is automatically set each time the item is saved.')
    )
    owner = models.ForeignKey(
        User,
        blank=True,
        null=True,
    )
    last_author = models.ForeignKey(
        User,
        blank=True,
        null=True,
        related_name='post_last_author'
    )
    primary_category = models.ForeignKey(
        Category,
        blank=True,
        null=True,
        related_name="primary_modelbase_set",
    )

    featured = models.BooleanField(
        _('Feature in latest'),
        default=False,
        help_text=_(
            'If checked this post will be displayed on the homepage '
            'under the latest section.'))

    featured_in_category = models.BooleanField(
        _('Feature in category post listing'),
        default=False,
        help_text=_(
            'If checked this post will be displayed in the category\'s '
            'list of featured posts on the homepage.'))

    related_posts = SortedManyToManyField('self', blank=True)

    localisation = models.ForeignKey(
        Localisation, blank=True, null=True)
    source = models.ForeignKey(
        'self',
        blank=True,
        null=True)
    position = models.PositiveIntegerField(
        _('Position in Ordering'), null=True, blank=True, default=0)

    def save(self, *args, **kwargs):
        # use django slugify filter to slugify
        if not self.slug:
            self.slug = slugify(self.title)

        # set created time to now if not already set.
        if not self.created_at:
            self.created_at = timezone.now()

        super(Post, self).save(*args, **kwargs)

    def __unicode__(self):  # pragma: no cover
        if self.subtitle:
            return '%s - %s' % (self.title, self.subtitle)
        else:
            return self.title


@receiver(post_save, sender=ContentRepository)
def auto_save_content_repository_to_git(sender, instance, created, **kwargs):
    workspace = EG.workspace(settings.GIT_REPO_PATH,
                             index_prefix=settings.ELASTIC_GIT_INDEX_PREFIX)
    workspace.sm.store_data(
        'LICENSE', instance.get_license_text(), 'Specify license.')


@receiver(post_save, sender=Post)
def auto_save_post_to_git(sender, instance, created, **kwargs):
    data = {
        "title": instance.title,
        "subtitle": instance.subtitle,
        "slug": instance.slug,
        "description": instance.description,
        "content": instance.content,
        "created_at": instance.created_at.isoformat(),
        "modified_at": instance.modified_at.isoformat(),
        # TODO: We should migrate this to localisation everywhere
        "language": (
            instance.localisation.get_code()
            if instance.localisation else None),
        "featured_in_category": instance.featured_in_category,
        "featured": instance.featured,
        "position": instance.position,
        "linked_pages": [related_post.uuid
                         for related_post in instance.related_posts.all()],
        "primary_category": (
            instance.primary_category.uuid
            if instance.primary_category
            else None),
        "source": (
            instance.source.uuid
            if instance.source
            else None),
    }

    # NOTE: If newly created always give it the highest ordering position
    if created:
        Post.objects.exclude(pk=instance.pk).update(position=F('position') + 1)

    # TODO: Setting the author on a commit is currently not supported
    #       by elastic-git
    #       See: https://github.com/universalcore/elastic-git/issues/23
    # author = utils.get_author_from_user(instance.last_author)

    try:
        workspace = EG.workspace(
            settings.GIT_REPO_PATH,
            index_prefix=settings.ELASTIC_GIT_INDEX_PREFIX)
        [page] = workspace.S(eg_models.Page).filter(uuid=instance.uuid)
        original = page.get_object()
        updated = original.update(data)
        workspace.save(updated, 'Page updated: %s' % instance.title)
        workspace.refresh_index()
    except ValueError:
        page = eg_models.Page(data)
        workspace.save(page, 'Page created: %s' % instance.title)
        workspace.refresh_index()
        # Always update the UUID as we've just generated a new one.
        Post.objects.filter(pk=instance.pk).update(uuid=page.uuid)


@receiver(post_delete, sender=Post)
def auto_delete_post_to_git(sender, instance, **kwargs):
    # TODO: Allow author information to be set in EG.
    # author = utils.get_author_from_user(instance.last_author)
    workspace = EG.workspace(settings.GIT_REPO_PATH,
                             index_prefix=settings.ELASTIC_GIT_INDEX_PREFIX)
    [page] = workspace.S(eg_models.Page).filter(uuid=instance.uuid)
    workspace.delete(
        page.get_object(), 'Page deleted: %s' % (instance.title,))
    workspace.refresh_index()


@receiver(post_save, sender=Category)
def auto_save_category_to_git(sender, instance, created, **kwargs):

    data = {
        "title": instance.title,
        "subtitle": instance.subtitle,
        "slug": instance.slug,
        "position": instance.position,
        "language": (
            instance.localisation.get_code()
            if instance.localisation else None),
        "featured_in_navbar": instance.featured_in_navbar,
        "source": (
            instance.source.uuid
            if instance.source else None),
    }

    # TODO: Not yet implemented
    # author = utils.get_author_from_user(instance.last_author)

    workspace = EG.workspace(settings.GIT_REPO_PATH,
                             index_prefix=settings.ELASTIC_GIT_INDEX_PREFIX)
    try:
        # FIXME: This can fail if we ever store a value with None
        #        as the uuid.
        [category] = workspace.S(eg_models.Category).filter(uuid=instance.uuid)
        original = category.get_object()
        updated = original.update(data)
        workspace.save(updated, 'Category updated: %s' % instance.title)
        workspace.refresh_index()
    except (GitCommandError, ValueError):
        category = eg_models.Category(data)
        workspace.save(category, 'Category created: %s' % instance.title)
        workspace.refresh_index()
        Category.objects.filter(pk=instance.pk).update(uuid=category.uuid)


@receiver(post_delete, sender=Category)
def auto_delete_category_to_git(sender, instance, **kwargs):
    # TODO: Not yet implemented
    # author = utils.get_author_from_user(instance.last_author)
    workspace = EG.workspace(settings.GIT_REPO_PATH,
                             index_prefix=settings.ELASTIC_GIT_INDEX_PREFIX)
    [category] = workspace.S(eg_models.Category).filter(uuid=instance.uuid)
    original = category.get_object()
    workspace.delete(original, 'Category deleted: %s' % instance.title)
    workspace.refresh_index()
